// Certified: 11/22/2006

/* ALLOCATES SPACE FOR ALL THE GLOBAL ARRAYS USED IN THE PROGRAM */

#include "ElectronHead.h"

void AllocateSpace(){
  int i,j;

  /*--------------------------------------------------------------------  
    SOME OF THESE ARRAYS REMAIN FIXED (F)  AFTER THEY ARE INITIALIZED, 
    WHILE SOME OTHERS CHANGE (C)   
    
    Gamma[N1+1]           (F)  THE ELECTRON LORENTZ FACTORS       
    GammaDot[N1+1]        (C)  THE TIME DERIVATIVE OF Gamma
    DeltaGamma[N1]        (F)  THE SPACING BETWEEN Gamma
    N[N1+1]               (C)  THE ELECTRON ENERGY DISTRIBUTION
    Q[N1]                 (F)  THE INJECTION OF ELECTRONS 
    Gamma_times_N[N1+1]   (C)  THE PRODUCT OF GAMMA * N   
    Gamma_square[N1+1]    (C)   THE SQUARE OF Gamma */
  
 Gamma=calloc(N1+1,sizeof(double));
 GammaDot=calloc(N1+1,sizeof(double));
 DeltaGamma=calloc(N1,sizeof(double)); 
 N=calloc(N1+1,sizeof(double));   
 Q=calloc(N1,sizeof(double));
 Gamma_times_N=calloc(N1+1,sizeof(double));
 Gamma_square=calloc(N1+1,sizeof(double));

 /* es[N2]         (F)  THE DISCRETE PHOTON ENERGIES
    loges          (F)  THE LOGARITHM OF es
    Synch[N2]      (C)  THE SYNCHROTRON POWER
    EC[N2]         (C)  THE EC POWER
    SSC[N2]        (C)  THE SSC POWER
    Efirst[N2]     (C)  THE FIRST ORDER SSC POWER
    Photons[N2]    (C)  THE TOTAL PHOTON ENERGY DENSITY IN THE SOURCE 
    
    gamma_lower_index[N2] (F)  THE INDEX OF THE SMALLEST Gamma
                               THAT IS LARGER THAN A GIVEN es[N1]

    EXTseed_upper  (F)         THE INDEX OF THE LARGEST ENTRY IN THE EXTERNAL
                               PHOTON FIELD THAT IS SMALLER THAN es[v]

    THE FOUR (C) ARRAYS ARE UPDATED ONCE EVERY TIME STEP */

 es=calloc(N2,sizeof(double));   
 loges=calloc(N2,sizeof(double));
 Synch=calloc(N2,sizeof(double));
 EC=calloc(N2,sizeof(double)); 
 SSC=calloc(N2,sizeof(double)); 
 Efirst=calloc(N2,sizeof(double));
 Photons=calloc(N2,sizeof(double)); 
 EXTseed_upper=calloc(N2,sizeof(double)); 
 gamma_lower_index=calloc(N2,sizeof(int));

 /* SRCphotonint  (C) THE INTEGRATED SOURCE PHOTON ENERGY DENSITY UP TO THE 
    CORRESPONDING ENERGY es.

    SRCfotonia[N2] (C)  THE PRODUCT OF THE SOURCE PHOTON 
    ENERGY DENSITY TIMES THE PHOTON ENERGY. 

    THEY ARE BOTH  CALCULATED ONCE PER TIME STEP THROUGH A CALL OF THE 
    Photon_int.c */

 SRCphotonint=calloc(N2,sizeof(double));
 SRCfotonia=calloc(N2,sizeof(double));

 /* ThomsonupperSRC[N1]  (F)  THE INDEX OF THE SOURCE PHOTON FIELD ENERGY UP 
    TO WHICH THE SCATTERING IS IN THE THOMSON REGIME FOR A GIVEN Gamma[N1]

    KNlowerSRC[N1]       (F)  THE INDEX OF THE SOURCE PHOTON FIELD ENERGY ABOVE
    WHICH THE SCATTERING IS IN THE DEEP KN  REGIME FOR A GIVEN Gamma[N1]
    THIS IS **NOT** USED IN THE CONTINUOUS LOSSES VERSION OF THE CODE.
    
    KNupperSRC[N1]       (F)  THE INDEX OF THE SOURCE PHOTON FIELD ENERGY UP TO
    WHICH THE PHOTON ENERGY IS LESS THAN Gamma[N1]

    ALL THESE ARE INITIALIZED IN Initialize.c
*/

 ThomsonupperSRC=calloc(N1+1,sizeof(int));
 KNlowerSRC=calloc(N1+1,sizeof(int));
 KNupperSRC=calloc(N1+1,sizeof(int));

 /* EXTe[N3]    (F)  THE EXTERNAL PHOTON FIELD ENERGIES  
    Uext[N3]    (F)  THE EXTERNAL PHOTON FIELD ENERGY DENSITY AT EACH ENERGY
    logEXTe[N3] (F)  THE LOG OF EXTe[N3]
    
    ThomsonupperEXT[N1+1]  (F)  THE INDEX OF THE EXTERNAL PHOTON FIELD ENERGY 
    UP TO WHICH SCATTERING IS IN THE THOMSON REGIME FOR A GIVEN Gamma[N1+1],

    KNlowerEXT[N1]  (F)   THE INDEX OF THE EXTERNAL PHOTON FIELD ENERGY  
    ABOVE WHICH THE SCATTERING IS IN THE DEEP KN  REGIME FOR A GIVEN Gamma[N1]
    THIS IS **NOT** USED IN THE CONTINUOUS LOSSES VERSION OF THE CODE. */

 EXTe=calloc(N3,sizeof(double));   
 Uext=calloc(N3,sizeof(double)); 
 logEXTe=calloc(N3,sizeof(double));
 ThomsonupperEXT=calloc(N1+1,sizeof(int));
 KNlowerEXT=calloc(N1+1,sizeof(int));
 
 /*  EXTphotonint[N3] (F)  THE INTEGRATED EXTERNAL PHOTON ENERGY DENSITY 
     UP TO THE  ENERGY OF THE INDEX.
      
     EXTfotonia[N3]  (f) THE PRODUCT OF THE EXTERNAL PHOTON 
     ENERGY DENSITY TIMES THE PHOTON ENERGY. 

     THEY ARE BOTH  CALCULATED ONLY ONCE IN Initialize.c */
 
 EXTphotonint=calloc(N3,sizeof(double));
 EXTfotonia=calloc(N3,sizeof(double));

 /* IN THE CALCULATION OF THE CONTINOUS LOSSES THERE IS AN INTEGRAL OVER 
    THE FINAL ELECTRON ENERGY. THIS INTEGRAL IS A FUNCTION OF THE INITIAL
    ELECTRON ENERGY AND THE INITIAL PHOTON ENERGY AND IS CALCULATED OLNY ONCE 
    IN  Initialize.c 
    FOR THIS, WE CREATED  **ContlossesSRC, and 
    **ContlossesEXT, TWO 2D ARRAYS,  THEIR FIRST INDEX CORRESPONDING TO THE 
    ELECTORN ARRAY SIZE (N1+1) AND THE SECOND ONE TO THE PHOTON ARRAY SIZE
    (N2 FOR THE SOURCE PHOTONS AND N3 FOR THE EXTERNAL PHOTONS) */ 
    
 ContlossesSRC=calloc(N1+1,sizeof(double*));

for (i=0;i<=N1;++i)
    ContlossesSRC[i]=calloc(N2,sizeof(double));

 ContlossesEXT=calloc(N1+1,sizeof(double*));

 for (i=0;i<=N1;++i)
   ContlossesEXT[i]=calloc(N3,sizeof(double));

/* EXTseed_lower (SRCseed_lower) IS A TWO DIMENSIONAL INTEGER ARRAY THAT 
   STORES THE INDEX OF THE MINIMUM EXT (SRC) SEED PHOTON ENERGY, 
   AS A FUNCTION OF THE ELECTRON ENERGY Gamma[j] AND FINAL PHOTON ENERGY es[i].
   NOTE THAT NOT THE WHOLE ARRAY IS FULL, BECAUSE ONLY SITUATIONS WITH 
   gamma[j]>es[i] ARE PLAUSIBLE*/

 EXTseed_lower=calloc(N1+1,sizeof(int*));
 
 for (i=0;i<=N1;++i)
   EXTseed_lower[i]=calloc(N2,sizeof(int));

 SRCseed_lower=calloc(N1+1,sizeof(int*));
 
 for (i=0;i<=N1;++i)
   SRCseed_lower[i]=calloc(N2,sizeof(int));

/* Csrc (Cext) IS A 3D ARRAY THAT HOLDS THE VALUES OF AN EXPRESSION USED 
   HEAVILY FOR THE CALCULATION OF THE COMPTON EMISSIVITIES. IT IS A FUNCTION 
   OF THE ENERGIES OF THE ELECTRON, THE INCOMING AND OUTGOING PHOTONS.
   THERE ARE TWO OF THOSE ARRAYS, ONE FOR THE EXTERNAL FIELD AND ONE FOR THE
   SOURCE FIELD*/

 Csrc=calloc(N2,sizeof(double**));
 for (i=0;i<N2;++i){
   Csrc[i]=calloc(N1+1,sizeof(double*));
   for (j=0;j<=N1;++j)
     Csrc[i][j]=calloc(N2,sizeof(double));
 }

 if (Csrc==NULL){
   fprintf(stderr,"Out of memory at line %d, file %s\n",__LINE__,__FILE__);
   exit(-1);
 }

 Cext=calloc(N2,sizeof(double**));
 for (i=0;i<N2;++i){
       Cext[i]=calloc(N1+1,sizeof(double*));
   for (j=0;j<=N1;++j)
     Cext[i][j]=calloc(N3,sizeof(double));
 }
 
 if (Cext==NULL){
   fprintf(stderr,"Out of memory at line %d, file %s\n",__LINE__,__FILE__);
   exit(-1);
 }
 printf("Done with allocating array space \n");
   return;
}
