// Certified: 11/22/2006

/*                        Central.c

CALCULATE THE ELECTRON DISTRIBUTION, THE EMISSIVITIES AND ADVANCE THE 
SOLUTION IN TIME WITH AN ADAPTIVE TIMESTEP, UP TO THE POINT WHERE THE 
SOLUTION HAS CONVERGED TO A STEADY-STATE. AT EACH TIME STEP WRITE
THE RESULTS IN A SERIES OF FILES TO BE FOUND IN THE DIRECTORY ../datafiles. */

#include "ElectronHead.h"

void Central()
{
  int i=0,j,v;
  FILE *ofp1,*ofp2,*ofp3,*ofp4,*ofp5,*ofp6,*ofp7,*ofp8,*ofp9;
  double *Ntemp=calloc(N1+1,sizeof(double));
  double *Ndiff=calloc(N1,sizeof(double));
  double aux,Ndiffmin,Ndiffmax,totaltime=0;
  
 //FILES TO STORE THE RESULTS

  // ELECTRON LORENTZ FACTORS
  
  ofp8=fopen("../datafiles/GammaResults","w");
             
  // OBSERVATION FREQUENCIES
                                 
  ofp9=fopen("../datafiles/EsResults","w");                

  
  for(j=0;j<=N1;++j)
    fprintf(ofp8,"%e\n",Gamma[j]);
    
  for(v=0;v<N2;v++)
    fprintf(ofp9,"%e\n",es[v]);

  fclose(ofp8);
  fclose(ofp9);


  Ntemp[N1]=0;  
 
  /*TIME EVOLUTION OF THE SYSTEM WITH AN ADAPTIVE TIMESTEP, SET BY THE 
    REQUIREMENT THAT THE FRACTIONAL CHANGE OF THE MAXIMUM RATIO OF ELECTRON 
    DENSITIES BETWEEN TWO TIMESTEPS IS n_ratio_max. WHEN THIS RESULTS TO 
    TIME STEPS OF SIZE GREATER t_steady, THE SYSTEM HAS REACH STEADY STATE */
  
  while (timestep <=t_steady)
    {
      totaltime=totaltime+timestep;
           printf("---------STEP %d\n",i);
      i++;
     
      /* CALCULATE A SERIES OF N2-1 INTEGRALS OF THE PHOTON ENERGY DENSITY
      INTEGRATING THE TABULATED VALUES OF "Photons" FROM THE LOWEST ENERGY 
      UP TO THE ith ENERGY. THIS WILL BE USED TO CALCULATE THE THOMSON REGIME
      LOSSES. THIS INTEGRATION IS PERFORMED HERE BECAUSE IT DOES 
      NOT DEPEND ON THE ELECTRON ENERGY GAMMA AND IT SAVES CPU TIME, BEING DONE
      ONLY ONCE PER TIMESTEP*/

      Photon_int();

      /*CALCULATE THE PRODUCT Gamma_times_N*/

      for(j=N1;j>=0;--j)
	Gamma_times_N[j]=Gamma[j]*N[j];
      
      /* CALCULATE THE ELECTRON ENERGY LOSS RATE FOR ALL Gamma BY CALLING THE 
	 FUNCTION Gamma_Dot, AND STORE THE RESULTS IN THE ARRAY GammaDot*/ 

      for(j=N1;j>=0;--j)
	GammaDot[j]=Gamma_Dot(j);

      /* CALCULATE THE ELECTRON DISTRIBUTION AS IT RESULTS FROM THE ELECTRON 
	 ENERGY LOSSES. THE RESULT IS CONTAINED IN THE ARRAY Ntemp[j]*/
      
      for(j=N1-1;j>=0;--j)
	CalcDistr(Ntemp,j);   

      //CALCULATE THE SYNCHROTRON EMISSION

      for(v=0;v<SynchIndex;++v)
	{
	  Synch[v]=Synchrotron(v);

	}
    
      for(v=0;v<N2;++v)
	{
	  //CALCULATE THE SSC EMISSION
	  SSC[v]=SSCemission(v);
	  
	  //CALCULATE THE EC EMISSION
	  EC[v]=ExtCompton(v);
	  
	  //ADD THEM UP
	  
	  Photons[v]=(Synch[v]+SSC[v]+EC[v]);
	  
	  //CALCULATE THE FIRST ORDER SSC
	  Efirst[v]=FirstCompton(v);
	  
	  
	  //CONVERT LUMINOSITY TO ENERGY DENSITY
	  
	  Photons[v]*=1./(4.*PI*pow(R,2.)*CI);
	  
	}


      /*---------------------------------------------------------------------
	CALCULATION OF THE NEXT timestep. THIS IS SET TO THE VALUE THAT WILL
	GIVE AN ESTIMATED CHANGE OF THE MAXIMUM FRACTIONAL DIFFERENCE BETWEEN
	N AND Dtemp BY n_ratio_max */

      Ndiffmax=fabs(N[jmin]-Ntemp[jmin])/Ntemp[jmin];
      for(j=jmin;j<N1;++j)   
	{                                  
	  Ndiff[j]=fabs(N[j]-Ntemp[j])/Ntemp[j];
	  if (Ndiff[j] > Ndiffmax)
	    Ndiffmax=Ndiff[j];
	}
      timestep=timestep*n_ratio_max/Ndiffmax;
      

          printf("EED maximum fractional change, timestep (R/c units) %12.4e%12.4e\n",Ndiffmax,timestep*CI/R);
      
      /*---------------------------------------------------------------------
	PASS THE CLACULATED  DISTRIBUTION Ntemp TO N*/
      for(j=0;j<=N1;++j)   
	{                                  

	  N[j]=Ntemp[j]; 
	}      
    }


  /*WRITE THE FINAL TIMESTEP COMOVING RESULTS THAT WILL BE USED 
    BY THE sed PROGRAM TO CALCULATE THE BEAMED EMISSION */

  ofp1=fopen("../datafiles/DistributionResults_source","w");       
  ofp2=fopen("../datafiles/SynchResults_source","w");              
  ofp3=fopen("../datafiles/SelfComptResults_source","w");         
  ofp4=fopen("../datafiles/FirstComptonResults_source","w");
  
  for(j=0;j<=N1;++j)   
	{                                  
	  fprintf(ofp1,"%e\n",N[j]);   
	}
 
 for(v=0;v<N2;++v)
	{
	  fprintf(ofp2,"%e\n",Synch[v]);  
	  fprintf(ofp3,"%e\n",SSC[v]);
	  fprintf(ofp4,"%e\n",Efirst[v]);
	}

  fclose(ofp1);
  fclose(ofp2); 
  fclose(ofp3);
  fclose(ofp4);

  free(Ntemp);
  free(Ndiff);
  printf("Done with the calculation \n");

  return;
}
