// Certified: 11/22/2006

/*                     Gamma_Dot.c 

CALCULATES THE ELECTRON ENERGY LOSSES gamma_dot FOR AN ELECTRON OF A GIVEN 
LORENTZ FACTOR gamma[J] IN THE CONTINUOUS APPROXIMATION.  
THE CALCULATION IS SPLIT IN TWO PARTS:
1. THE THOMSON LOSSES 
2. THE KLEIN NISHINA LOSSES		   */

#include "ElectronHead.h"

double Gamma_Dot(int j)

{
  double gamma_dot,UEXT=0.,USRC=0.,gammaprimemin,
  *SRCaux=calloc(N2,sizeof(double)),srcKNlosses=0,
    *EXTaux=calloc(N3,sizeof(double)),extKNlosses=0;
  int i;

  /*--------------------------------------------------------------------------
                                 THOMSON REGIME LOSSES 
    			 				 
UEXT IS THE ENERGY DENSITY OF THE PART OF THE EXTERNAL PHOTON FIELD THAT
IS IN THE THOMSON REGIME FOR THE CURRENT Gamma[j]. 
RECALL THAT ThomsonupperEXT [J] IS THE INDEX OF THE HIGHEST EXTERNAL 
PHOTON ENERGY THAT IS STILL IN THE THOMSON REGIME FOR THE Gamma[J] UNDER
CONSIDERATION. */

  UEXT=EXTphotonint[ThomsonupperEXT[j]];

/* SIMILAR CONSIDERATIONS APPLY FOR USRC, THIS TIME FOR THE 
     SOURCE PHOTON FIELD. */
  
  USRC=SRCphotonint[ThomsonupperSRC[j]];

// THE THOMSON PART OF gamma_dot IS 

  gamma_dot=-Const1*Gamma[j]*Gamma[j]*(UB+UEXT+USRC);

/*--------------------------------------------------------------------------
                        KLEIN-NISHINA REGIME LOSSES
A. THE SOURCE PHOTONS

HERE WE INTEGRATE, OVER THE PART OF THE SOURCE SEED PHOTONS THAT ARE IN THE 
KN REGIME, THE PRODUCT OF  Photons[i]*ContlossesSRC[j][i], 
WHERE Photons[i] IS THE  SOURCE PHOTON ENERGY  DENSITY AND 
ContlossesSRC[j][i] IS AN INTEGRAL CALCULATED IN Initialize.c.  
MORE ON THAT ON THE DOCUMENTATION AND THE PAPER.

WE  DO THE INTEGRATION LOGARITHMICALLY: IN THIS CASE THE INTEGRANT IS 
Photons[i]*es[i]*ContlossesSRC[j][i]. 
THE PRODUCT OF THE FIRST TWO TERMS HAS BEEN CALCULATED IN photon_int.c 
AND HAS BEEN STORED IN THE ARRAY SRCfotonia. 

THE INTEGRANT IS CALCULATED AND STORED IN THE ARRAY SCRaux. 
THEN, A TABULATED INTEGRATION IS PERFORMED, WHICH RESULTS
TO THE PART OF  gamma_dot DUE TO  SOURCE PHOTONS THAT ARE IN THE 
KLEIN-NISHINA REGIME*/
  
  for(i=ThomsonupperSRC[j]+1;i<=KNupperSRC[j];++i)
    SRCaux[i]=SRCfotonia[i]* ContlossesSRC[j][i];
     
  srcKNlosses=-tab_int(SRCaux,esnorm,ThomsonupperSRC[j]+1,KNupperSRC[j]); 
 
  // B. THE EXTERNAL PHOTONS, REPEAT THE SAME PROCEDURE. 

  for(i=ThomsonupperEXT[j]+1;i<N3;++i)
    EXTaux[i]=EXTfotonia[i] * ContlossesEXT[j][i];
     
  extKNlosses=-tab_int(EXTaux,EXTnorm,ThomsonupperEXT[j]+1,N3-1); 
 
  // ADD THE CONTRIBUTIONS UP.

  gamma_dot=gamma_dot+srcKNlosses+extKNlosses;
  
  free(SRCaux);
  free(EXTaux);

  return gamma_dot;
}
 
