// Certified: 11/22/2006

/*                            Initialize.c
   
                INITIALIZE ARRAYS AND CONSTANTS               */

#include "ElectronHead.h"


                    
void Initialize ()
{
  double temp1,temp2,temp4,K,temp5,eth,ekn,gprimemin,beta,aux,
    constant1,lowEXT=1.e-5,highEXT=2.e1,Ue,BBpeak=2.821,k1,min_eseed,bbulk;
  int i,j,n,v,k,ThupperEXT,ThupperSRC,kn_lowerEXT,kn_lowerSRC,kn_upperSRC;

  
  /*INITIALIZE es, THE ARRAY THAT HOLDS THE PHOTON ENERGIES
    (IN UNITS OF ELECTRON REST MASS) IN WHICH THE CALCULATION OF THE EMISSION 
    IS PERFORMED. IT ALSO CALCULATES SynchIndex, THE INDEX OF THE MAXIMUM 
    ENERGY AT WHICH SYNCHROTRON ENERGY IS EMITTED. */
  
  CalcEs();

  //THE ELECTRON ESCAPE TIME FROM THE SYSTEM IN SECONDS

  tesc*=R/CI;
  
  //A CONSTANT USED IN THE CALCULATION OF THE CONTINUOUS LOSSES IN Gamma_Dot

  Const1=4.*(SigmaT)/(Me*CI*3.);
 
  //MAGNETIC FIELD ENERGY DENSITY IN erg /

  UB=B*B/8./PI;

  //A CONSTANT USED IN THE CALCULATION OF THE COMPTON EMISSION

  Const2=2.*PI*pow(Re,2.)*CI;

  //-----------------------------------------------------------------------

  /*POPULATE THE Gamma ARRAY WITH LOGARITHMICALLY SPACED ELECTRON  ENERGIES.
    INITIALIZE THE ARRAY Gamma_square. 
    CALCULATE THE INDEX jmin OF GammaMin*/
  
  jmin=0;
  temp1=log10((double)GammaMax/Gamma0);
  
  for(j=0;j<=N1;++j)
    {
      Gamma[j]=Gamma0*pow(10,temp1*((double)j/(N1-1)));
      Gamma_square[j]=Gamma[j]*Gamma[j];
      if (Gamma[j]<GammaMin)
	jmin=j+1;
    }

  /*  DeltaGamma  THE SPACING BETWEEN Gamma  */
  
  for(j=0;j<N1;++j)
    DeltaGamma[j]=Gamma[j+1]-Gamma[j];

    /*    INITIALIZE THE ELECTRON INJECTION ARRAY Q */
 
  switch(Injection)
    {
    case 1: //POWER LAW INJECTION 
      if(s!=2.0)
	k1=Luminosity*(2.-s)/(Me*pow(CI,2.)*(pow(GammaMax,2.-s)-pow(GammaMin,2.-s)));
      else
	k1=Luminosity/(Me*pow(CI,2.)*log(GammaMax/GammaMin));
      
      for(j=0;j<N1;++j)
	{
	  if (Gamma[j] >  GammaMin && Gamma[j] < GammaMax/10.)
	    Q[j]=(k1*pow(Gamma[j],(-s)));
	  else if (Gamma[j]<GammaMin)
	    Q[j]=0.;
		else if (Gamma[j]>GammaMax/10.)
		Q[j]=(k1*pow(GammaMax/10.,1.7)*pow(Gamma[j],(-s-1.7)));
	}
      break;
      
    case 2: //DELTA FUNCTION INJECTION
      k1=Luminosity/(Me*CI*CI*GammaMax);
      
      for(j=0;j<N1;++j)
	{
	  if(fabs(Gamma[j]-GammaMax)<DeltaGamma[N1-2]/10.)
	    Q[j]=k1/DeltaGamma[j];
	  else
	    Q[j]=0.;
	}
      break;
    }
   
  //-----------------------------------------------------------------------
  /* THE EXTERNAL PHOTON FIELD CAN  BE THE COSMIC MICROWAVE BACKGROUND
     AT A GIVEN REDSHIFT, OR ANY OTHER BLACKBODY EXTERNAL PHOTON FIELD.
     THE VARIABLE PhotonField IS 0 IN THE CMB CASE, 1 FOR OTHER BB */   
  	
  //A: NO BULK MOTION
  if (Gbulk == 1.0)
    {
        //CMB PEAK PHOTON ENERGY
        if (PhotonField==0)
	epsilon0=BBpeak*Kb*Tcmb*(Z+1.)/(Me*pow(CI,2)); 
  
      /*INITIALIZE THE ENERGIES OF THE EXTERNAL PHOTON FIELD, GOING FROM
	epsilon0*lowEXT UP TO  epsilon0*highEXT. */
      
      EXTe[0]=lowEXT*epsilon0;
      EXTe[N3-1]=highEXT*epsilon0;
      temp1=log10(EXTe[N3-1]/EXTe[0])/(N3-1);
      
      logEXTe[0]=log(EXTe[0]);

      for(v=1;v<N3-1;++v){
	EXTe[v]=EXTe[0]*pow(10,(temp1*v));
	logEXTe[v]=log(EXTe[v]);
      }
  
      /*INITIALIZE THE ENERGY DENSITY OF THE EXTERNAL PHOTON FIELD*/
  
      temp1=8.*PI*pow(Me,4.)*pow(CI,5.)/pow(Planck,3.);
      
      if (PhotonField==0){
	Ue=alpha*pow(Tcmb*(Z+1.),4.);
	temp2=Me*pow(CI,2.)/(Kb*Tcmb*(Z+1.));
	for(v=0;v<N3;++v)
	  Uext[v]=temp1*pow(EXTe[v],3.)/(exp(temp2*EXTe[v])-1.);
      }else{
	Ue=U0;
	for(v=0;v<N3;++v)
	  Uext[v]=temp1*pow(EXTe[v],3.)/(exp(EXTe[v]*BBpeak/epsilon0)-1.) 
	    *U0/(pow(Me*CI*CI*epsilon0/BBpeak/Kb,4.)*alpha);
      }
    }

  
  //B: BULK MOTION WITH LORENTZ FACTOR Gbulk
  if (Gbulk != 1.0)
    {
      bbulk=pow((1.-1./(Gbulk*Gbulk)),.5);
      
      if (PhotonField==0)
	epsilon0=BBpeak*Kb*Tcmb*(Z+1.)/(Me*pow(CI,2.)); //CMB PHOTON ENERGY
      
      /*INITIALIZE THE ENERGIES OF THE EXTERNAL PHOTON FIELD, GOING FROM
	epsilon0*lowEXT*Gbulk*(1+bbulk) UP TO
	epsilon0*highEXT*Gbulk*(1+bbulk). */
      
      EXTe[0]=lowEXT*epsilon0*Gbulk*(1+bbulk);
      EXTe[N3-1]=highEXT*epsilon0*Gbulk*(1+bbulk);
      temp1=log10(EXTe[N3-1]/EXTe[0])/(N3-1);
      
      logEXTe[0]=log(EXTe[0]);

      for(v=1;v<N3-1;++v){
	EXTe[v]=EXTe[0]*pow(10,(temp1*v));
	logEXTe[v]=log(EXTe[v]);	
      }
  
      /*INITIALIZE THE ENERGY DENSITY OF THE EXTERNAL PHOTON FIELD*/
  
      temp1=4.*PI*pow(Me,4.)*pow(CI,5.)/pow(Planck,3.);
      
      if (PhotonField==0){
	Ue=alpha*pow(Tcmb*(1.+Z),4.);
	temp2=Me*pow(CI,2.)/(Kb*Tcmb*(1.+Z));
	for(v=0;v<N3;++v)
	  Uext[v]=temp1*epsilon0/BBpeak/(bbulk*Gbulk)*pow(EXTe[v],2.)
	    *log(   (1.-exp(-EXTe[v]*Gbulk*(1.+bbulk)*BBpeak/epsilon0))  
		    /(1.-exp(-EXTe[v]*BBpeak/(epsilon0*Gbulk*(1.+bbulk))))); 
      }else{
	Ue=U0;
	for(v=0;v<N3;++v)
	  Uext[v]=temp1*epsilon0/BBpeak/(bbulk*Gbulk)*pow(EXTe[v],2.)
	    *log(
		 (1.-exp(-EXTe[v]*Gbulk*(1.+bbulk)*BBpeak/epsilon0))  
		 /(1.-exp(-EXTe[v]*BBpeak/(epsilon0*Gbulk*(1.+bbulk))))
		 )
	    *U0/(pow(Me*CI*CI*epsilon0/BBpeak/Kb,4.)*alpha); 
              }   
    }
 	
  //aux=0.;

  //for(v=0;v<N3;++v){
  //aux=aux+(logEXTe[2]-logEXTe[1])*Uext[v]*EXTe[v];
	//printf("%8d%12.4e%12.4e\n",v,EXTe[v],Uext[v]*EXTe[v]);
  //	}
  //aux=aux-(logEXTe[2]-logEXTe[1])*Uext[0]*EXTe[0]/2.-(logEXTe[2]-logEXTe[1])*Uext[N3-1]*EXTe[N3-1]/2.;
// printf("%12.4e\n",aux);

  //-------------------------------------------------------------------------  
  /*INITIALIZE THE TIMESTEP ASSUMING SYNCHROTRON, EC, AND TOTAL INJECTED 
    ELECTRONS CONVERTED TO PHOTON ENERGY DENSITY IN THE SOURCE, ALL 
    IN THE THOMSON REGIME.  THIS TIMESTEP IS THE COOLING TIME OF
    THE LOWEST ENERGY ELECTRON. THIS IS THE LARGEST CHARACTERISTIC TIME OF 
    THE SYSTEM, AND IT IS USED TO BRING THE SYSTEM UP TO A STATE WHERE 
    INJECTION HAS BEEN TAKING PLACE FOR TIME LONG ENOUGHT THAT EVEN THE
    LOWEST ENERGY ELECTRONS HAD TIME TO COOL. THIS IS OK BECAUSE OUR NUMERICAL
    SCHEME CAN TAKE ARBITRARILY LARGE STEPS.  THE SUBSEQUENT TIME EVOLUTION 
    WILL BRING THE SYSTEM TO ITS STEADY STATE. */
  
  timestep=3.*Me*CI/((pow(B,2.)/(8.*PI)+Ue*Gbulk*Gbulk+Luminosity/(4.*PI*R*R*CI))*4.*SigmaT*Gamma0);

  /*WHEN THE ADAPTIVE TIMESTEP BECOMES GREATER THAN t_steady, 
    STEADY STATE HAS BEEN REACHED. WE SET THIS TO BE 10^9 TIMES THE ESCAPE 
    TIME, WHICH IS MORE THAN SUFFICIENT. */

  t_steady=1.e9*tesc;

 //-------------------------------------------------------------------------

  //LOGARITHMIC SPACING BETWEEN ARRAY ELEMENTS USED IN INTEGRATIONS

  esnorm=log(es[2]/es[1]);
  gammanorm=log(Gamma[2]/Gamma[1]);
  EXTnorm=log(EXTe[2]/EXTe[1]);


 //A CONSTANT USED IN THE CALCULATION OF THE SYNCHROTRON EMISSIVITY

    Const3=pow(Qe,3.)*B/Planck;
  
/* --------------------------------------------------------------------------
     CALCULATION OF A SERIES OF N3-1 INTEGRALS OF THE EXTERNAL PHOTON FIELD
     ENERGY DENSITY UP TO THE ENERGY CORRESPONDING TO INDEX V. THIS INTEGRATION
     IS PERFORMED ONCE, BECAUSE THE EXTERNAL PHOTON FIELD DOES NOT CHANGE 

     NOTE THAT BECAUSE OF THE LARGE DYNAMIC RANGE BETWEEN THE LOWER AND UPPER
     LIMITS OF MANY OF THE INTEGRALS PERFORMED IN THIS CODE, MOST INTEGRATIONS 
     ARE DONE LOGARITHMICALLY, THAT IS, THROUGH A SUBSTITUTION OF x BY log(x).
     THIS REQUIRES A MULTIPLICATION OF THE INTEGRAND BY x */

  for(v=0;v<N3;++v)
    {  
      EXTfotonia[v]=Uext[v]*EXTe[v]; 
      EXTphotonint[v]=tab_int(EXTfotonia,EXTnorm,0,v);
    }

    /* ------------------------------------------------------------------------
     FOR EVERY Gamma, FIND THE INDEX OF THE UPPER LIMIT OF THE EXTERNAL(SOURCE)
     PHOTON ARRAY THAT IS STILL IN THE THOMSON REGIME AND STORE IT IN 
     ThomsonupperEXT  (ThomsonupperSRC) INTEGER ARRAY. 
     THESE ARE  USED IN  Gamma_Dot.c 

     FIND THE INDEX OF THE LOWER LIMIT OF THE EXTERNAL (SOURCE) PHOTON ARRAY
     THAT  IS STILL  IN THE DEEP KN  REGIME AND STORE IN KNlowerEXT 
     (KNlowerSRC) INTEGER ARRAY. THESE ARE NOT USED IN THE CONTINUOUS 
     ENERGY LOSS APPROXIMATION CODE.

     FIND THE INDEX THAT CORRESPONDS TO THE MAXIMUM SRC PHOTON ENERGY THAT IS 
     LESS THAN Gamma[j] AND STORE IN KNupperSRC.
     THIS IS USED IN Gamma_Dot.c */

  for(j=0;j<=N1;++j)
    {
      /* eth IS THE PHOTON ENERGY BELOW WHICH WE APPROXIMATE  THE 
	 SCATTERING AS THOMSON SCATTERING */
 
      eth=Thomsonlimit/Gamma[j];
      ThupperEXT=1;

      while(EXTe[ThupperEXT]< eth && ThupperEXT<N3)
      	++ThupperEXT;

      ThomsonupperEXT[j]=ThupperEXT-1;

      ThupperSRC=1;

      while(es[ThupperSRC]<eth && ThupperSRC<N2)
      	++ThupperSRC;

      ThomsonupperSRC[j]=ThupperSRC-1;
         
      /* ekn IS THE PHOTON ENERGY ABOVE  WHICH WE APPROXIMATE  THE 
	 SCATTERING AS DEEP KN SCATTERING */

      ekn=KNlimit/Gamma[j];
      kn_lowerEXT=0;
      
      while(EXTe[kn_lowerEXT]<ekn && kn_lowerEXT<N3-1)
      	++kn_lowerEXT;

      KNlowerEXT[j]=kn_lowerEXT;

       kn_lowerSRC=0;
      
      while(es[kn_lowerSRC]<ekn && kn_lowerSRC<N2-1)
      	++kn_lowerSRC;

      KNlowerSRC[j]=kn_lowerSRC;
      
      /* KNupperSRC IS THE INDEX OF THE HIGHEST SRC PHOTON ENERGY BELOW
	 Gamma[j]. PHOTONS OF HIGHER ENERGIES ARE NOT CONSIDERED. */
      
      kn_upperSRC=0;
      
      while(es[kn_upperSRC]<Gamma[j] && kn_upperSRC<N2-1)
	++kn_upperSRC;

      --kn_upperSRC;
      
      KNupperSRC[j]=kn_upperSRC;
    }
  //-----------------------------------------------------------------------

  /* FOR EVERY SRC PHOTON ENERGY es[i], FIND THE INDEX j OF THE SMALLEST
     Gamma[j] THAT IS GREATER THAN es[i]: es[i]<Gamma[j]. STORE IT IN
     THE ARRAY gamma_lower_index[i]. THIS IS USED IN THE INTEGRATION LIMITS
     FOR THE CALCULATION OF THE INVERSE COMPTON EMISSIVITIES  */

  for (i=0;i<N2;++i){
    j=0;
    while (es[i]> Gamma[j] && (j < N1) )
      j++;
    gamma_lower_index[i]=j;
  }

  /* FOR EVERY SRC PHOTON ENERGY es[i], FIND THE INDEX j OF THE GREATEST
     EXTe[j] THAT IS SMALLER THAN es[i]: EXTe[j]<es[i]. STORE IT IN
     THE ARRAY EXTseed_upper[i]. THIS IS USED IN THE CALCULATION
     OF THE INVERSE COMPTON EMISSIVITIES  */

  for (i=0;i<N2;++i){
    j=1;
    while ((es[i]>EXTe[j]) && j <N3)
      j++;
    EXTseed_upper[i]=j-1;
  } 
  
  //-----------------------------------------------------------------------
  
  /* EXTseed_lower (SRCseed_lower) IS A TWO DIMENSIONAL INTEGER ARRAY THAT 
   STORES THE INDEX OF THE MINIMUM EXT (SRC) SEED PHOTON ENERGY, 
   AS A FUNCTION OF THE ELECTRON ENERGY Gamma[j] AND FINAL PHOTON ENERGY es[i].
   NOTE THAT NOT THE WHOLE ARRAY IS FULL, BECAUSE ONLY SITUATIONS WITH 
   gamma[j]>es[i] ARE PLAUSIBLE*/
  
  for (i=0;i<N2;++i){	       
    for (j=gamma_lower_index[i];j<N1;++j){
      min_eseed=es[i]/(4.*Gamma[j]*(Gamma[j]-es[i]));
      n=0;
      while (min_eseed >= EXTe[n]  && n<N3-1)
	n++;
      EXTseed_lower[j][i]=n;
    }
  }

  for (i=0;i<N2;++i){	       
    for (j=gamma_lower_index[i];j<N1;++j){
      min_eseed=es[i]/(4.*Gamma[j]*(Gamma[j]-es[i]));
      n=0;
      while (min_eseed >= es[n]  && n<N2-1)
	  n++;
      SRCseed_lower[j][i]=n;
    }
  }

  /*--------------------------------------------------------------------
    CALCULATION OF AN INTEGRAL OVER THE FINAL ELECTRON ENERGY FOR A GIVEN 
     INITIAL ELECTRON AND INITIAL PHOTON ENERGY. THIS IS USED
     IN CALCULATING THE CONTINUOUS ENERGY LOSSES THAT ARE NOT IN THE 
     THOMSON REGIME. IT IS DONE TWICE ONCE FOR THE SOURCE PHOTON FIELD AND 
     ONCE FOR THE EXTERNAL PHOTON FIELD, AND THE  RESULTS ARE STORED IN 
     ContlossesSRC and ContlossesEXT. 
     THESE ARE TWO  2D arrays, THEIR FIRST INDEX CORRESPONDS TO THE 
     ELECTRON ARRAY (SIZE N1+1) AND THE SECOND ONE TO THE PHOTON ARRAYS
     (SIZE N2 FOR THE SRC PHOTONS AND SIZE N3 FOR THE EXT PHOTONS)
     
     NOTE THAT THE ARRAYS ARE NOT FULL, BECAUSE FOR A GIVEN Gamma[j], WE 
     HAVE TWO CONSTRAINTS:
     THE FIRST IS THAT WE DO NOT WANT TO CONSIDER
     PHOTONS THAT ARE IN THE THOMSON REGIME, WE ARE ONLY INTERESTED IN THE
     PARTS OF THE PHOTON ARRAYS WITH INDEXES   > ThomsonupperSRC[j] AND
     > ThomsonupperEXT[j]. 
     THE SECOND IS THAT THE SEED PHOTONS MUST HAVE ENERGY LESS 
     THAN Gamma[j]. SO, FOR A GIVEN j, THE LAST ELEMENT OF ContlossesSRC[j][i]
     IS i=KNupperSRC[j]. 
     THE LAST ELEMENT OF ContlossesEXT[j][i] IS i=N3-1, BECAUSE WE ASSUME THAT 
     THE EXTERNAL PHOTON FIELD IS CONFINED TO DIMENSIONLESS ENERGIES<1, 
     SO IN THIS CASE, THIS CONSTRAINT IS NOT RELEVANT */

  //SOS: Things will change for discrete losses  
  
  constant1=3*SigmaT/(4.*Me*CI);
 
  //----> LOOP OVER Gamma[j] STARTS

  for (j=0;j<=N1;++j){ 
    gaux=Gamma[j]; /* THIS IS USED TO PASS THE Gamma[j] VALUE TO c.c 
		      gaux IS A GLOBAL VARIABLE DEFINED IN ElectronHead.h */

    beta=pow((1.-1./Gamma[j]/Gamma[j]),.5); //THIS IS  beta=u/c 
     
    //----> LOOP OVER es[i] STARTS

    for (i=ThomsonupperSRC[j]+1;i<=KNupperSRC[j];++i){
      
      esaux=es[i];  /* THIS IS USED TO PASS THE es[i] VALUE TO c.c  
		       esaux IS A GLOBAL VARIABLE DEFINED IN ElectronHead.h */
      
      // THIS IS THE LOWER LIMIT OF THE INTEGRATION 
      
      gprimemin=Gamma[j]+es[i]-(Gamma[j]*Gamma[j]*es[i]*(1.+beta)*(1.+beta))/
	(1.+2.*Gamma[j]*es[i]*(1.+beta)); 

      ContlossesSRC[j][i]=constant1/(esaux*esaux*gaux*gaux)                                              *gaussint64(c,gprimemin,gaux);
      
    }//----> LOOP OVER es[i] ENDS

     //----> LOOP OVER EXTe[i] STARTS

    for (i=ThomsonupperEXT[j]+1;i<N3;++i){
      esaux=EXTe[i]; 

      gprimemin=Gamma[j]+esaux-(Gamma[j]*Gamma[j]*esaux*(1.+beta)*(1.+beta))/
	(1.+2.*Gamma[j]*esaux*(1.+beta));

      ContlossesEXT[j][i]=constant1/(esaux*esaux*gaux*gaux)                                              *gaussint64(c,gprimemin,gaux);

    }//----> LOOP OVER EXTe[i] ENDS
  } //----> LOOP OVER Gamma[j] ENDS
 
 /*--------------------------------------------------------------------------
   FILL THE 3D ARRAYS Csrc AND Cext. THEIR INDEXES v,j,k, CORRESPOND TO
   THE FINAL PHOTON, THE ELECTRON, AND THE SEED PHOTON. THIS IS THE "Q" OR "C" 
   EXPRESSION SEEN FREQUENTLY IN COMPTON EMISSIVITY CALCULATIONS*/

  for (v=0;v<N2;++v){
    for(j=gamma_lower_index[v];j<N1;++j){
      
      if(SRCseed_lower[j][v]>=v)
	continue;
      
      temp1=4.*Gamma[j];
      temp2=es[v]/(temp1*(Gamma[j]-es[v])); 
      temp5=log(temp2);
      
      for(k=SRCseed_lower[j][v];k<=v;++k){
	K=temp2/es[k];
	temp4=temp1*es[k]*K;
	
	Csrc[v][j][k]=(2.*K*(temp5-loges[k])+(1.+2.*K)*(1.-K)+
		       (temp4*temp4*(1.-K)/
			(2.*(1.+temp4))))/es[k];
      }
    }
  }
  
  for (v=0;v<N2;++v){

    if(gamma_lower_index[v]>=N1)
      continue;
   
    // EXTseed_upper[v]  IS THE INDEX OF THE MAXIMUM SEED PHOTON ENERGY

    if(EXTseed_upper[v]<=0)
      continue;
    
    for(j=gamma_lower_index[v];j<N1;++j){
       
      if(EXTseed_lower[j][v]>=N3-1)
	continue;
     
      if(EXTseed_upper[v]<=EXTseed_lower[j][v])
	continue;

      temp1=4.*Gamma[j];
      temp2=es[v]/(temp1*(Gamma[j]-es[v]));
      temp5=log(temp2);

      for(k=EXTseed_lower[j][v];k<EXTseed_upper[v];++k){
	K=temp2/EXTe[k];
	
	temp4=temp1*EXTe[k]*K;
	Cext[v][j][k]=(2.*K*(temp5-logEXTe[k])+(1.+2.*K)*(1.-K)+
		       (temp4*temp4*(1.-K)/
			(2.*(1.+temp4))))/EXTe[k] ;
      }
    }
    
  }
  
 printf("Done with initializing parameters and arrays \n");
  

 return;
}
