// Certified: 11/20/2006

/*---------------------------------------------------------------------------
 A HOMOGENEOUS SOURCE: CALCULATION OF THE STEADY-STATE ELECTRON DISTRIBUTION
         
       By Markos Georganopoulos, Brian Wingert, Demos Kazanas, 
	 Reynaldo Castro, Eric Perlman          
                                                                  
       								     
An electron distribution is injected into a homogeneous source of radius  
R, characterized by a magnetic field B. The source is permeated by 
an external photon field, and is moving relativistically with a bulk 
Lorentz factor of our choice.

The electrons suffer synchrotron, external Compton (EC) and synchrotron   
-self Compton (SSC) losses. These losses are taken into account,
and the electron energy distribution (EED) as a function of time is calculated,
following an implicit numerical scheme. The program stops when the EED
has reached a steady state.
---------------------------------------------------------------------------*/


/* INCLUDE THE HEADER FILE ElectronHead.h. IT DEFINES PHYSICAL CONSTANTS 
   AND ALL THE FUNCTIONS USED FOR THE CALCULATION. */


#include "ElectronHead.h"

int main(void){

  long begin;
  begin=clock();
  
/* READ THE PHYSICAL AND COMPUTATIONAL PARAMETERS REQUIRED FROM THE FILE 
   "../datafiles/Initial_Values" */
 
  Initial_Values();

 /* ALLOCATE SPACE FOR ALL THE GLOBAL ARRAYS USED IN THE PROGRAM */
 
  AllocateSpace();

  /* INITIALIZE ARRAYS AND CONSTANTS */

  Initialize(); 
                                
 
 /* CALCULATE THE ELECTRON DISTRIBUTION, THE EMISSIVITIES AND ADVANCE THE 
    SOLUTION IN TIME WITH AN ADAPTIVE TIMESTEP, UP TO THE POINT WHERE THE 
    SOLUTION HAS CONVERGED TO A STEADY-STATE. AT EACH TIME STEP WRITE
    THE RESULTS IN A SERIES OF FILES TO BE FOUND IN THE DIRECTORY ../datafiles.
    ROUTINES FOR PLOTTING THE DATA CAN BE FOUND IN THE DIRECTORY idlprograms*/

  Central(); 
 
 /* FREE ALLOCATED SPACE */

 FreeSpace();
 
 // PRINT  ON THE SCREEN THE TIME IN SECONDS USED
 
 printf("CPU SECONDS: %ld\n",(clock()-begin)/CLOCKS_PER_SEC); 
 
 return  ;
}
