/* THIS PROGRAM CALCULATES THE OBSERVED SED USING THE EED CALCULATED FROM THE 
   PROGRAM eed.*/

// Certified: 02/16/2007

#include "ElectronHead.h"

int main(void){ 
  double aux,min_eseed,BBpeak=2.821,lowEXT=1.e-3,highEXT=2.e1,temp1,Ue,
    temp2,temp5,K,temp4;
  int j,i,n,v,k; 
  FILE *ifp1,*ifp2,*ifp3,*ifp4,*ofp1,*ofp2,*ofp3,*ofp4;
 

  Const2=2.*PI*pow(Re,2.)*CI;

/* READ THE PHYSICAL AND COMPUTATIONAL PARAMETERS REQUIRED FROM THE FILE 
   "../datafiles/Initial_Values" */


  Initial_Values();

  
 /* ALLOCATE SPACE FOR ALL THE GLOBAL ARRAYS USED IN THE PROGRAM */
 
  AllocateSpace();

 

  /*------------------------------------------------------------------
    FIRST, TRANSFORM THE EED IN THE OBSERVER'S FRAME.
    THIS IS NEEDED TO CALCULATE THE EXTERNAL COMPTON EMISSION.
    READ THE Gamma VALUES FROM THE FILE ../datafiles/GammaResults 
    AND THE EED (ARRAY N) VALUES FROM THE FILE  
    ../datafiles/DistributionResults_source AND BEAM THE RESULTS */

  ifp1=fopen ("../datafiles/GammaResults","r");
  ifp2=fopen ("../datafiles/DistributionResults_source","r");
 
  for (j=0;j<=N1;++j){
   fscanf(ifp1,"%lf",&aux);
   Gamma[j]=aux*delta;
   fscanf(ifp2,"%lf",&aux);
   N[j]=aux*pow(delta,3.);
   Gamma_times_N[j]=Gamma[j]*N[j];
   Gamma_square[j]=Gamma[j]*Gamma[j];
  }
  
  fclose(ifp1);
  fclose(ifp2);
  gammanorm=log(Gamma[2]/Gamma[1]);
 
  
  /*------------------------------------------------------------------
    NOW, BEAM THE SYNCHROTRON, SSC, AND FIRST ORDER SSC EMISSION TO THE
    OBSERVER'S FRAME. 
    READ THE es VALUES FROM THE FILE ../datafiles/EsResults,
    THE Synch VALUES FROM THE FILE datafile/SynchResults_source,
    THE SSC VALUES FROM THE FILE datafile/SelfComptResults_source,
    THE Efirst VALUES FROM THE FILE datafile/FirstComptonResults_source
    AND BEAM THE RESULTS */

  ifp1=fopen ("../datafiles/EsResults","r");
  ifp2=fopen ("../datafiles/SynchResults_source","r");
  ifp3=fopen ("../datafiles/SelfComptResults_source","r");
  ifp4=fopen ("../datafiles/FirstComptonResults_source","r");

  
  for (i=0;i<N2;++i){
    fscanf(ifp1,"%lf",&aux);
    es[i]=aux*delta;
     fscanf(ifp2,"%lf",&aux);
     Synch[i]=aux*pow(delta,3);
     fscanf(ifp3,"%lf",&aux);
     SSC[i]=aux*pow(delta,3);
     fscanf(ifp4,"%lf",&aux);
     Efirst[i]=aux*pow(delta,3);
  }

  fclose(ifp1);
  fclose(ifp2);
  fclose(ifp3);
  fclose(ifp4);

  /* WRITE THE BEAMED (OBSERVED) QUANTITIES IN FILES */
 
  ofp1=fopen("../datafiles/Es","w");    
  ofp2=fopen("../datafiles/Synchrotron","w"); 
  ofp3=fopen("../datafiles/SSC","w");
  ofp4=fopen("../datafiles/SSC1","w");
  
for (i=0;i<N2;++i){  
     fprintf(ofp1,"%e\n",es[i]);  
     fprintf(ofp2,"%e\n",Synch[i]);
     fprintf(ofp3,"%e\n",SSC[i]);
     fprintf(ofp4,"%e\n",Efirst[i]);
}

  fclose(ofp1);
  fclose(ofp2);
  fclose(ofp3);
  fclose(ofp4);

  /*-----------------------------EC emissivity----------------------------
    START WITH PRELIMINARIES THAT ARE USED IN THE INTEGRATION. THESE ARE 
    TAKEN FROM THE FILE Initialize.c USED IN THE eed CODE.*/



  /*-----------------------------------------------------------------------
  INITIALIZE THE EXTERNAL PHOTON FIELD. THIS CAN EITHER BE THE COSMIC 
  MICROWAVE BACKGROUND AT A GIVEN REDSHIFT, OR ANY OTHER BLACKBODY EXTERNAL 
  PHOTON FIELD. THE VARIABLE PhotonField IS 0 IN THE CMB CASE, 1 OTHERWISE.
  NOTE THAT THIS IS THE PHOTON FIELD IN THE LOCAL FRAME OF REFERENCE, 
  NOT THE COMOVING FRAME OF THE FLOW.*/   
  
  
        //CMB PEAK PHOTON ENERGY
        if (PhotonField==0)
	epsilon0=BBpeak*Kb*Tcmb*(Z+1)/(Me*pow(CI,2)); 
  
      /*INITIALIZE THE ENERGIES OF THE EXTERNAL PHOTON FIELD, GOING FROM
	epsilon0*lowEXT UP TO  epsilon0*highEXT. */
      
      EXTe[0]=lowEXT*epsilon0;
      EXTe[N3-1]=highEXT*epsilon0;
      temp1=log10(EXTe[N3-1]/EXTe[0])/(N3-1);
      
      logEXTe[0]=log(EXTe[0]);

      for(v=1;v<N3-1;++v){
	EXTe[v]=EXTe[0]*pow(10,(temp1*v));
	logEXTe[v]=log(EXTe[v]);
      }
  
      /*INITIALIZE THE ENERGY DENSITY OF THE EXTERNAL PHOTON FIELD*/
  
      temp1=8.*PI*pow(Me,4.)*pow(CI,5.)/pow(Planck,3.);
      
      if (PhotonField==0){
	Ue=alpha*pow(Tcmb*(Z+1.),4.);
	temp2=Me*pow(CI,2.)/(Kb*Tcmb*(Z+1.));
	for(v=0;v<N3;++v)
	  Uext[v]=temp1*pow(EXTe[v],3.)/(exp(temp2*EXTe[v])-1.);
      }else{
	Ue=U0;
	for(v=0;v<N3;++v)
	  Uext[v]=temp1*pow(EXTe[v],3.)/(exp(EXTe[v]*BBpeak/epsilon0)-1.) 
	    *U0/(pow(Me*CI*CI*epsilon0/BBpeak/Kb,4.)*alpha);
      }
  
   
      EXTnorm=log(EXTe[2]/EXTe[1]);


  /* FOR EVERY SRC PHOTON ENERGY es[i], FIND THE INDEX j OF THE SMALLEST
     Gamma[j] THAT IS GREATER THAN es[i]: es[i]<Gamma[j]. STORE IT IN
     THE ARRAY gamma_lower_index[i]. THIS IS USED IN THE INTEGRATION LIMITS
     FOR THE CALCULATION OF THE INVERSE COMPTON EMISSIVITIES  */

  for (i=0;i<N2;++i){
    j=0;
    while (es[i]> Gamma[j]  && (j < N1))
      j++;
    gamma_lower_index[i]=j;
  }

 /* FOR EVERY SRC PHOTON ENERGY es[i]  FIND THE INDEX j OF THE GREATEST
     EXTe[j] THAT IS SMALLER THAN es[i]: EXTe[j]<es[i]. STORE IT IN
     THE ARRAY EXTseed_upper[i]. THIS IS USED IN THE CALCULATION
     OF THE INVERSE COMPTON EMISSIVITIES  */

  for (i=0;i<N2;++i){
    j=1;
    while ((es[i]>EXTe[j]) && j <N3)
      j++;
    EXTseed_upper[i]=j-1;

  } 

 /* EXTseed_lower IS A TWO DIMENSIONAL INTEGER ARRAY THAT 
   STORES THE INDEX OF THE MINIMUM EXT SEED PHOTON ENERGY, 
   AS A FUNCTION OF THE ELECTRON ENERGY Gamma[j] AND FINAL PHOTON ENERGY es[i].
   NOTE THAT NOT THE WHOLE ARRAY IS FULL, BECAUSE ONLY SITUATIONS WITH 
   gamma[j]>es[i] ARE PLAUSIBLE*/
  
  for (i=0;i<N2;++i){	       
    for (j=gamma_lower_index[i];j<N1;++j){
      min_eseed=es[i]/(4.*Gamma[j]*(Gamma[j]-es[i]));
      n=0;
      while (min_eseed >= EXTe[n]  && n<N3-1)
	n++;
      EXTseed_lower[j][i]=n;
    }
  }

 /*--------------------------------------------------------------------------
   FILL THE 3D ARRAYS Csrc AND Cext. THEIR INDEXES v,j,k, CORRESPOND TO
   THE FINAL PHOTON, THE ELECTRON, AND THE SEED PHOTON. */

  for (v=0;v<N2;++v){
    
    if(gamma_lower_index[v]>=N1)
      continue;
   
    // EXTseed_upper[v]  IS THE INDEX OF THE MAXIMUM SEED PHOTON ENERGY

    if(EXTseed_upper[v]<=0)
      continue;
    
    for(j=gamma_lower_index[v];j<N1;++j){
       
      if(EXTseed_lower[j][v]>=N3-1)
	continue;
     
      if(EXTseed_upper[v]<=EXTseed_lower[j][v])
	continue;

      temp1=4.*Gamma[j];
      temp2=es[v]/(temp1*(Gamma[j]-es[v]));
      temp5=log(temp2);

      for(k=EXTseed_lower[j][v];k<EXTseed_upper[v];++k){
	K=temp2/EXTe[k];

	temp4=temp1*EXTe[k]*K;
	Cext[v][j][k]=(2.*K*(temp5-logEXTe[k])+(1.+2.*K)*(1.-K)+
		       (temp4*temp4*(1.-K)/
			(2.*(1.+temp4))))/EXTe[k] ;
      }
    }
  
  }
  //FINALLY, CALCULATE THE EC EMISSION

  ofp1=fopen("../datafiles/EC","w");    
  for(v=0;v<N2;++v)
    {
      EC[v]=ExtCompton(v);
      fprintf(ofp1,"%e\n",EC[v]);  
    }
  fclose(ifp1);

  /* FREE ALLOCATED SPACE */

  FreeSpace();
  
  return 0.;
}
